#include <stdint.h>
#include <stdbool.h>

// C_Meter Click
sbit C_METER_RST  at GPIO_PIN1_bit;
sbit C_METER_INT at GPIO_PIN3_bit;

void system_setup( void );
void setup_interrupt( void );
void InitTimerA( void );

volatile bool flag;
volatile uint32_t start_val;
volatile uint32_t end_val;
char uart_text[ 40 ];
char final_text[ 40 ];
uint32_t int_count = 0;
uint32_t timer_count_end = 0;
bool sec_flag = false;
volatile float c_cal = 0.0;
volatile float t_meas = 0.0;
volatile float c_meas = 0.0;
volatile float final = 0.0;
volatile float t_cal = 0.0;


void main()
{
    //Local Declarations
    bool calibration_flag = true;

    system_setup();
    setup_interrupt();
    InitTimerA();

    t_cal = 1.0 / 11278.0;
    c_cal = t_cal / 77616.0;

    int_count = 0;

    while(1)
    {

        if( sec_flag )
        {
            //Debugging
            sprintf( uart_text, "Interrupts : %u\r\n", timer_count_end );
            UART1_Write_Text( uart_text );
            //Measurment
            t_meas = 1.0 / timer_count_end;
            c_meas = t_meas / 77616.0;
            final = fabs( c_cal - c_meas );
            //Print out
            UART1_Write_Text( "Calibrated: " );
            FloatToStr( c_cal, uart_text );
            UART1_Write_Text( uart_text );
            UART1_Write_Text( "\r\n" );
            UART1_Write_Text( "Final: " );
            FloatToStr( final, final_text );
            UART1_Write_Text( final_text );
            UART1_Write_Text( "\r\n" );
            //Reset flags
            sec_flag = false;
            int_count = 0;
            timer_count_end = 0;
        }
    }
}

void system_setup( void )
{
    //GPIO
    GPIO_Digital_Output(&GPIO_PORT_00_07, _GPIO_PINMASK_1 );
    GPIO_Digital_Input(&GPIO_PORT_00_07, _GPIO_PINMASK_3 );

    //RST Toggle
    C_METER_RST = 0;
    Delay_ms(50);
    C_METER_RST = 1;

    //UART
    UART1_Init( 9600 );
    UART1_Write_Text( "UART Initialized\r\n" );

}

void setup_interrupt( void )
{
// Enable digital input on PORT_24
 // GPIO_Digital_Input(&GPIO_PORT_24_31,_GPIO_PINMASK_0);
// Enable all interrupts
  IRQ_CTRL.B31 = 0;
// Set rising edge to detect interrupt
  //GPIO5_CFG0_bit = 1;
  GPIO03_CFG0_bit = 1;
// Interrupt capable
  //GPIO24_CFG1_bit = 1;
  GPIO03_CFG1_bit = 1;
// Set GPIO direction as input
  //GPIO24_CFG2_bit = 0;
  //GPIO24_CFG3_bit = 0;
  GPIO03_CFG2_bit = 0;
  GPIO03_CFG3_bit = 0;
// Enable interrupt on PORT_24
  //GPIO_INT_EN_bit = 1;
  GPIO_INT_EN_00_31bits.GPIO_INT_EN3 = 1;
}

void interrupt() iv IVT_GPIO_IRQ{
  Delay_50us();

// Clear interrupt flag of PORT_24
    GPIO_INT_PEND_00_31bits.GPIO_INT_PEND3 = 1;
    int_count++;
}

void InitTimerA() 
{
  TIMER_CONTROL_0 = 2;
  TIMER_SELECT = 0;
  TIMER_PRESC_LS = 36;
  TIMER_PRESC_MS = 244;
  TIMER_WRITE_LS = 63;
  TIMER_WRITE_MS = 6;
  TIMER_CONTROL_3 = 0;
  TIMER_CONTROL_4 |= 17;
  TIMER_CONTROL_2 |= 16;
  TIMER_INT |= 2;
  TIMER_CONTROL_1 |= 1;
}
void TimerInterrupt() iv IVT_TIMERS_IRQ{
  //Enter your code here
  if (TIMER_INT_A_bit)
  {
      TIMER_INT = (TIMER_INT & 0xAA) | (1 << 0);
      timer_count_end = int_count;
      int_count = 0;
      sec_flag = true;
  }
}